#
# coding=utf-8
"""test_attrdict - Test AttrDict support"""
# Copyright © 2014-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from unittest import TestCase

from expecter import expect

from rdial.utils import AttrDict


class AttrDictTest(TestCase):
    def setUp(self):
        self.ad = AttrDict(carrots=3, snacks=0)

    def test_base(self):
        expect(self.ad).isinstance(dict)

        expect(self.ad['carrots']) == 3
        expect(self.ad['snacks']) == 0

        expect(sorted(self.ad.keys())) == ['carrots', 'snacks']

    def test___contains__(self):
        expect(self.ad).contains('carrots')
        expect(self.ad).does_not_contain('prizes')

    def test___getattr__(self):
        expect(self.ad.carrots) == 3
        expect(self.ad.snacks) == 0

    def test___setattr__(self):
        self.ad.carrots, self.ad.snacks = 0, 3
        expect(self.ad.carrots) == 0
        expect(self.ad.snacks) == 3

    def test___delattr__(self):
        expect(self.ad).contains('carrots')
        del self.ad['carrots']
        expect(self.ad).does_not_contain('carrots')



#
# coding=utf-8
"""i18n - Internationalisation support for rdial."""
# Copyright © 2011-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

import gettext

from os import path

from .compat import PY2


kwargs = {
    'localedir': path.join(path.realpath(path.dirname(__file__)), 'locale'),
    'names': ['gettext', 'ngettext', ]
}
if PY2:
    kwargs['unicode'] = 1

gettext.install('rdial', **kwargs)

_, N_ = _, ngettext  # NOQA: bindings from gettext.install above

__all__ = (_, N_)



#
# coding=utf-8
"""test_utils - Test utility functions"""
# Copyright © 2015-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from os import listdir
from subprocess import CalledProcessError
from sys import version_info
from time import sleep

from click.testing import CliRunner
from expecter import expect
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from rdial.utils import (AttrDict, check_output, newer, read_config,
                         remove_current, write_current)


def test_check_output():
    expect(check_output(['echo', 'hello'])) == 'hello\n'


def test_check_output_py26_compat():
    if version_info[:2] == (2, 6):
        expect(check_output(['echo', 'hello'])) == 'hello\n'
    else:
        with patch('subprocess.check_output', side_effect=AttributeError):
            expect(check_output(['echo', 'hello'])) == 'hello\n'


def test_check_output_py26_compat_fail():
    if version_info[:2] == (2, 6):
        with expect.raises(CalledProcessError):
            check_output(['false', ])
    else:
        with patch('subprocess.check_output', side_effect=AttributeError):
            with expect.raises(CalledProcessError):
                check_output(['false', ])


def test_read_config_local():
    conf = read_config('tests/data/local.ini')
    expect(conf['local test'].as_bool('read')) == True


def test_handle_current():
    runner = CliRunner()
    with runner.isolated_filesystem() as tempdir:
        globs = AttrDict(directory=tempdir)
        bare = lambda globs, task: True
        write_current(bare)(globs, task='test')
        expect(listdir(tempdir)).contains('.current')
        remove_current(bare)(globs, task='test')
        expect(listdir(tempdir)).does_not_contain('.current')
        # check idempotent...
        remove_current(bare)(globs, task='test')


def test_newer():
    runner = CliRunner()
    with runner.isolated_filesystem() as tempdir:
        f1 = open('%s/file1' % tempdir, 'w')
        sleep(0.1)
        f2 = open('%s/file2' % tempdir, 'w')
        expect(newer(f2.name, f1.name)) == True
        expect(newer(f1.name, f2.name)) == False
        expect(newer(f1.name, f1.name)) == False



#
# coding=utf-8
"""test_datetime - Test ISO-8601 handling"""
# Copyright © 2011-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from datetime import (datetime, timedelta)

from expecter import expect
from nose2.tools import params

from rdial.utils import (format_datetime, parse_datetime, parse_datetime_user)


@params(
    ('2011-05-04T08:00:00Z', datetime(2011, 5, 4, 8, 0)),
    ('2011-05-04T09:15:00Z', datetime(2011, 5, 4, 9, 15)),
)
def test_parse_datetime(string, expected):
    expect(parse_datetime(string)) == expected


@params(
    ('5 minutes ago', timedelta(minutes=5)),
    ('1 hour ago -5 minutes', timedelta(hours=1, minutes=5)),
)
def test_parse_datetime_via_date_command(string, delta):
    now = datetime.utcnow().replace(microsecond=0)
    # Accept a 2.5 second smudge window
    expect(parse_datetime_user(string)) >= now - delta
    expect(parse_datetime_user(string)) < now - delta + timedelta(seconds=2.5)


@params(
    (datetime(2011, 5, 4, 8, 0), '2011-05-04T08:00:00Z'),
    (datetime(2011, 5, 4, 9, 15), '2011-05-04T09:15:00Z'),
)
def test_format_datetime(dt, expected):
    expect(format_datetime(dt)) == expected



#
# coding=utf-8
"""test_setup_py - Test setup.py functionality"""
# Copyright © 2011-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from expecter import expect
from nose2.tools import params
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

import setup


@params(
    ('plain', ['tabulate', ]),
    ('recurse', ['click', 'tabulate', ]),
)
def test_parse_requires(input, expected):
    requires = setup.parse_requires('../tests/data/requires/%s.txt' % input)
    expect(requires) == expected


@params(
    ((2, 7, 6), ['unicodecsv', ]),
    ((3, 4, 2), []),
)
@patch('setup.sys')
def test_parse_markers(version, expected, setup_sys):
    setup_sys.version_info = version
    requires = setup.parse_requires('../tests/data/requires/markers.txt')
    expect(requires) == expected



#
# coding=utf-8
"""compat - Python 2/3 compatibility support for rdial."""
# Copyright © 2011-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from sys import version_info

PY2 = version_info[0] == 2

if PY2:  # pragma: Python 2
    def mangle_repr_type(klass):
        """Class decorator to force bytestrings for repr with Python 2.

        Args:
            klass (class): Class to patch
        """
        klass.__repr_unicode__ = klass.__repr__

        def wrapper(self):
            """Encode __repr_unicode__ as UTF-8.

            The use of UTF-8 can cause problems on some poorly configured
            machines, but better support for properly configured machines is
            far more important to me.

            Returns:
                str: UTF-8 encoded ``repr``
            """
            return self.__repr_unicode__().encode('utf-8')
        klass.__repr__ = wrapper
        return klass
else:  # pragma: Python 3
    mangle_repr_type = lambda x: x  # pylint: disable=invalid-name



#
# coding=utf-8
"""test_event_filter - Test event filter handling"""
# Copyright © 2011-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from expecter import expect
from nose2.tools import params

from rdial.events import Events


def test_fetch_events_for_task():
    events = Events.read('tests/data/test', write_cache=False)
    expect(len(events.for_task(task='task2'))) == 1


@params(
    ({'year': 2011, }, 2),
    ({'year': 2011, 'month': 1}, 1),
    ({'year': 2011, 'month': 3, 'day': 1}, 1),
    ({'year': 2011, 'month': 3, 'day': 31}, 0),
)
def test_fetch_events_for_date(date, expected):
    events = Events.read('tests/data/date_filtering', write_cache=False)
    expect(len(events.for_date(**date))) == expected


def test_fetch_events_for_week():
    events = Events.read('tests/data/date_filtering', write_cache=False)
    expect(len(events.for_week(year=2011, week=9))) == 1



#
# coding=utf-8
"""test_cmdline - Test command line functionality"""
# Copyright © 2011-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from click import BadParameter
from expecter import expect
from nose2.tools import params

from rdial.cmdline import (StartTimeParamType, TaskNameParamType)


@params(
    ('valid_name', True),
    ('also-valid-name', True),
    ('.invalid_name', BadParameter),
    ('valid.name', True),
    ('invalid/name', BadParameter),
    ('', BadParameter),
    ('x' * 256, BadParameter),
)
def test_task_name_validity(string, expected):
    p = TaskNameParamType()
    if expected is True:
        expect(p.convert(string, None, None) == string)
    else:
        with expect.raises(expected):
            p.convert(string, None, None)


@params(
    ('yesterday', True),
    ('', True),
    ('2011-05-04T09:15:00Z', True),
    ('AB1 time', BadParameter),
)
def test_start_time_validity(string, expected):
    p = StartTimeParamType()
    if expected is True:
        expect(p.convert(string, None, None) == string)
    else:
        with expect.raises(expected):
            p.convert(string, None, None)



#
# coding=utf-8
"""utils - Utility functions for rdial."""
# Copyright © 2012-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

import datetime
import functools
import os
import subprocess

import ciso8601
import click
import configobj

from . import compat


# Set up informational message functions
def _colourise(text, colour):
    """Colour text, if possible.

    Args:
        text (str): Text to colourise
        colour (str): Colour to display text in
    """
    click.termui.secho(text, fg=colour, bold=True)


def success(text):
    """Pretty print a success message.

    Args:
        text (str): Text to format
    """
    _colourise(text, 'green')


def fail(text):
    """Pretty print a failure message.

    Args:
        text (str): Text to format
    """
    _colourise(text, 'red')


def warn(text):
    """Pretty print a warning message.

    Args:
        text (str): Text to format
    """
    _colourise(text, 'yellow')


def safer_repr(obj):
    """Produce a repr string for an object.

    .. note::
        This exists solely for use on deep objects that can be expelled deep in
        the dependency libraries.  It should *not* be required for any
        serviceable objects.

    Args:
        obj (object): Object to produce repr for

    Returns:
        str: :func:`repr` output, or a fallback string
    """
    try:
        return repr(obj)
    except Exception:
        return '<invalid repr>'


class RdialError(ValueError):

    """Generic exception for rdial."""

    if not compat.PY2:  # pragma: Python 3
        @property
        def message(self):
            """Compatibility hack for Python 3."""
            return self.args[0]


class AttrDict(dict):

    """Dictionary with attribute access.

    See also:
        :obj:`dict`
    """

    def __contains__(self, key):
        """Check for item membership.

        Args:
            key (object): Key to test for

        Returns:
            bool: True, if item in dictionary
        """
        return hasattr(self, key) or super(AttrDict, self).__contains__(key)

    def __getattr__(self, key):
        """Support item access via dot notation.

        Args:
            key (object): Key to fetch
        """
        try:
            return self[key]
        except KeyError:
            raise AttributeError(safer_repr(key))

    def __setattr__(self, key, value):
        """Support item assignment via dot notation.

        Args:
            key (object): Key to set value for
            value (object): Value to set key to
        """
        try:
            self[key] = value
        except:
            raise AttributeError(safer_repr(key))

    def __delattr__(self, key):
        """Support item deletion via dot notation.

        Args:
            key (object): Key to delete
        """
        try:
            del self[key]
        except KeyError:
            raise AttributeError(safer_repr(key))


#: Map duration string keys to timedelta args
_MAPPER = {'D': 'days', 'H': 'hours', 'M': 'minutes', 'S': 'seconds'}


def parse_delta(string):
    """Parse ISO-8601 duration string.

    Args:
        string (str): Duration string to parse

    Returns:
        datetime.timedelta: Parsed delta object
    """
    if not string:
        return datetime.timedelta(0)
    parsed = {}
    block = []
    for c in string[1:]:
        if c.isalpha():
            if block:
                parsed[_MAPPER[c]] = int(''.join(block))
            block = []
        else:
            block.append(c)
    return datetime.timedelta(**parsed)  # pylint: disable=star-args


def format_delta(timedelta_):
    """Format ISO-8601 duration string.

    Args:
        timedelta_ (datetime.timedelta): Duration to process

    Returns:
        str: ISO-8601 representation of duration
    """
    if timedelta_ == datetime.timedelta(0):
        return ''
    days = '%dD' % timedelta_.days if timedelta_.days else ''
    hours, minutes = divmod(timedelta_.seconds, 3600)
    minutes, seconds = divmod(minutes, 60)
    hours = '%02dH' % hours if hours else ''
    minutes = '%02dM' % minutes if minutes else ''
    seconds = '%02dS' % seconds if seconds else ''
    return 'P%s%s%s%s%s' % (days, 'T' if hours or minutes or seconds else '',
                            hours, minutes, seconds)


def parse_datetime(string):
    """Parse datetime string.

    Args:
        string (str): Datetime string to parse

    Returns:
        datetime.datetime: Parsed datetime object
    """
    if not string:
        datetime_ = datetime.datetime.utcnow()
    else:
        datetime_ = ciso8601.parse_datetime(string[:19])
        if not datetime_:
            raise ValueError('Unable to parse timestamp %r'
                             % (safer_repr(string), ))
    return datetime_


def parse_datetime_user(string):
    """Parse datetime string from user.

    We accept the normal ISO-8601 formats, but kick through to the formats
    supported by the system's date command if parsing fails.

    Args:
        string (str): Datetime string to parse

    Returns:
        datetime.datetime: Parsed datetime object
    """
    try:
        datetime_ = parse_datetime(string)
    except ValueError:
        try:
            output = check_output(['date', '--utc', '--iso-8601=seconds', '-d',
                                   string])
            datetime_ = ciso8601.parse_datetime(output.strip()[:19])
        except subprocess.CalledProcessError:
            datetime_ = None
    if not datetime_:
        raise ValueError('Unable to parse timestamp %r'
                         % (safer_repr(string), ))
    return datetime_


def format_datetime(datetime_):
    """Format ISO-8601 datetime string.

    Args:
        datetime_ (datetime.datetime): Datetime to process

    Returns:
        str: ISO-8601 compatible string
    """
    return datetime_.strftime('%Y-%m-%dT%H:%M:%SZ')


def iso_week_to_date(year, week):
    """Generate date range for a given ISO-8601 week.

    ISO-8601 defines a week as Monday to Sunday, with the first week of a year
    being the first week containing a Thursday.

    Args:
        year (int): Year to process
        week (int): Week number to process

    Returns:
        tuple of datetime.date: Date range objects for given week
    """
    bound = datetime.date(year, 1, 4)
    iso_start = bound - datetime.timedelta(days=bound.isocalendar()[2] - 1)
    start = iso_start + datetime.timedelta(weeks=week - 1)
    end = start + datetime.timedelta(weeks=1)
    return start, end


def check_output(args, **kwargs):
    """Simple check_output implementation for Python 2.6 compatibility.

    Note:
        This hides stderr, unlike the normal check_output function.

    Args:
        args (list): Command and arguments to call

    Returns:
        str: Command output

    Raises:
        subprocess.CalledProcessError: If command execution fails
    """
    try:
        output = subprocess.check_output(args, stderr=subprocess.PIPE,
                                         **kwargs)
    except AttributeError:
        process = subprocess.Popen(args, stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE, **kwargs)
        output, _ = process.communicate()
        retcode = process.poll()
        if retcode:
            raise subprocess.CalledProcessError(retcode, args[0])
    if not compat.PY2:  # pragma: Python 3
        output = output.decode()
    return output


def read_config(user_config=None, cli_options=None):
    """Read configuration data.

    Args:
        user_config (str): User defined config file
        cli_options (dict): Command line options

    Returns:
        configobj.ConfigObj: Parsed configuration data
    """
    # Only base *must* exist
    conf = configobj.ConfigObj(os.path.dirname(__file__) + '/config',
                               file_error=True)
    conf['xdg_data_location'] = xdg_data_location()
    for string in os.getenv('XDG_CONFIG_DIRS', '/etc/xdg').split(':'):
        conf.merge(configobj.ConfigObj(string + '/rdial/config'))
    conf.merge(configobj.ConfigObj(xdg_config_location() + '/config'))
    conf.merge(configobj.ConfigObj(os.path.abspath('.rdialrc')))
    conf.merge(configobj.ConfigObj(user_config))

    if cli_options:
        cli_conf = ['[rdial]', ]
        cli_conf.extend("%s = %r" % (k, v) for k, v in cli_options.items()
                        if v is not None)
        conf.merge(configobj.ConfigObj(cli_conf))

    return conf


def write_current(fun):
    """Decorator to write :file:`current` file on function exit.

    See also:
        :doc:`/taskbars`

    Returns:
        types.FunctionType: Wrapped function
    """
    @functools.wraps(fun)
    def wrapper(*args, **kwargs):
        """Write value of ``task`` argument to ``current on exit.

        Args:
            args (tuple): Positional arguments
            kwargs (dict): Keyword arguments
        """
        globs = args[0]
        fun(*args, **kwargs)
        with click.open_file('%s/.current' % globs.directory, 'w') as f:
            f.write(kwargs['task'])
    return wrapper


def remove_current(fun):
    """Decorator to remove :file:`current` file on function exit.

    See also:
        :doc:`/taskbars`

    Returns:
        types.FunctionType: Wrapped function
    """
    @functools.wraps(fun)
    def wrapper(*args, **kwargs):
        """Remove ``current`` file on exit.

        Args:
            args (tuple): Positional arguments
            kwargs (dict): Keyword arguments
        """
        globs = args[0]
        fun(*args, **kwargs)
        if os.path.isfile('%s/.current' % globs.directory):
            os.unlink('%s/.current' % globs.directory)
    return wrapper


def newer(fname, reference):
    """Check whether given file is newer than reference file.

    Args:
        fname (str): File to check
        reference (str): file to test against

    Returns:
        bool: True if ``fname`` is newer than ``reference``
    """
    return os.stat(fname).st_mtime > os.stat(reference).st_mtime


def xdg_cache_location():
    """Return a cache location honouring $XDG_CACHE_HOME.

    Returns:
        str: Location of cache directory
    """
    user_dir = os.getenv('XDG_CACHE_HOME',
                         os.path.join(os.getenv('HOME', '/'), '.cache'))
    return os.path.join(user_dir, 'rdial')


def xdg_config_location():
    """Return a config location honouring $XDG_CONFIG_HOME.

    Returns:
        str: Location of config directory
    """
    user_dir = os.getenv('XDG_CONFIG_HOME',
                         os.path.join(os.getenv('HOME', '/'), '.config'))
    return os.path.join(user_dir, 'rdial')


def xdg_data_location():
    """Return a data location honouring $XDG_DATA_HOME.

    Returns:
        str: Location of data directory
    """
    user_dir = os.getenv('XDG_DATA_HOME', os.path.join(os.getenv('HOME', '/'),
                         '.local/share'))
    return os.path.join(user_dir, 'rdial')



#! /usr/bin/env python
# coding=utf-8
"""setup.py - Setuptools tasks and config for rdial"""
# Copyright © 2011-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

import imp
import sys

from setuptools import setup

# Hack to import _version file without importing rdial/__init__.py, its
# purpose is to allow import without requiring dependencies at this point.
with open('rdial/_version.py') as ver_file:
    _version = imp.load_module('_version', ver_file, ver_file.name,
                               ('.py', ver_file.mode, imp.PY_SOURCE))


def parse_requires(file):
    deps = []
    with open('extra/%s' % file) as req_file:
        entries = map(lambda s: s.split('#')[0].strip(), req_file.readlines())
    for dep in entries:
        if not dep or dep.startswith('#'):
            continue
        elif dep.startswith('-r '):
            deps.extend(parse_requires(dep.split()[1]))
            continue
        elif ';' in dep:
            dep, marker = dep.split(';')
            if not eval(marker.strip(),
                        {'python_version': '%s.%s' % sys.version_info[:2]}):
                continue
        deps.append(dep)
    return deps

install_requires = parse_requires('requirements.txt')

test_requires = parse_requires('requirements-test.txt')

with open('README.rst') as f:
    long_description = f.read()

if __name__ == '__main__':
    setup(
        name='rdial',
        version=_version.dotted,
        description='Simple time tracking for simple people',
        long_description=long_description,
        author='James Rowe',
        author_email='jnrowe@gmail.com',
        url='https://github.com/JNRowe/rdial',
        license='GPL-3',
        keywords='timetracking task report',
        packages=['rdial', ],
        include_package_data=True,
        package_data={'': ['config', 'rdial/locale/*/LC_MESSAGES/*.mo']},
        entry_points={'console_scripts': ['rdial = rdial.cmdline:main', ]},
        install_requires=install_requires,
        zip_safe=False,
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Environment :: Console',
            'Intended Audience :: Developers',
            'Intended Audience :: End Users/Desktop',
            'Intended Audience :: Other Audience',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved',
            'License :: OSI Approved :: GNU General Public License (GPL)',
            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
            'Natural Language :: English',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Topic :: Office/Business',
            'Topic :: Office/Business :: News/Diary',
            'Topic :: Office/Business :: Scheduling',
            'Topic :: Other/Nonlisted Topic',
            'Topic :: Utilities',
        ],
    )



#
# coding=utf-8
"""test_xdg_basedir - Test XDG base directory support"""
# Copyright © 2011-2016  James Rowe <jnrowe@gmail.com>
#
# This file is part of rdial.
#
# rdial is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# rdial is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# rdial.  If not, see <http://www.gnu.org/licenses/>.

from expecter import expect
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from rdial.utils import (xdg_config_location, xdg_data_location)


def test_config_no_args():
    with patch.dict('os.environ', {'XDG_CONFIG_HOME': '~/.xdg/config'}):
        expect(xdg_config_location()) == '~/.xdg/config/rdial'


def test_config_no_home():
    with patch.dict('os.environ', clear=True):
        expect(xdg_config_location()) == '/.config/rdial'


def test_data_no_args():
    with patch.dict('os.environ', {'XDG_DATA_HOME': '~/.xdg/local'}):
        expect(xdg_data_location()) == '~/.xdg/local/rdial'


def test_data_no_home():
    with patch.dict('os.environ', clear=True):
        expect(xdg_data_location()) == '/.local/share/rdial'
